/*================================================================
 * cgmres.c
 * Controller of NMPC C/GMRES
 *
 * The calling syntax is:
 *
 * [u, err] = cgmres(t, x, x1)
 *
 * Inputs t: previous time, x: prevoius state, x1: current state
 * Ouputs u: new input, err: error norm
 *
 * This is a MEX-file for MATLAB.
 * 
 * 2018.11.16 K.Hirota
 * Special Thanks, T.Iori
 * 
 * Copyright (c) 2018 Kazuki Hirota
 * Released under the MIT license
 * https://opensource.org/licenses/mit-license.php
 *===============================================================*/

#include "mex.h"
#include "matrix.h"
#include "cohover_xfnopre_GUI.c"	/* Rewrite the file name */

static int initialized = 0;

/* When you finish the mex function, clear memory by using a command below
 * clear function, clear mex, clear all, clear ('function name').
 * Caution: You can't clear memory and finish this function by only using command "clear".*/
void cLeanup(void) {
	if(initialized != 0) {

		 /* Delete Global Arrays */
		 freevector(tau);
		 freematrix(xtau);
		 freematrix(xtau1);
		 freematrix(ltau);
		 freematrix(utau);
		 freematrix(utau1);
		 freematrix(hutau);
		 freematrix(hutau1);
		 freematrix(hutau2);
		 freevector(x1s);
		 freevector(bvec);
		 freevector(duvec);
		 freevector(dutmp);
		 freevector(errvec);


		 initialized = 0;
		 mexPrintf("MEX-file is safely terminated.\n");

    }

}

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{	
	double t, *x, *x1;             /* Input scalar & matrices */
    double *u, *err;               /* Output matrices */

	int i, j;
	double hu2;	

    /* Check for proper number of arguments. */
    if(nrhs!=3) {
        mexErrMsgIdAndTxt("MyToolbox:arrayProduct:nrhs","Three inputs required.");
    }
    if(nlhs!=2) {
        mexErrMsgIdAndTxt("MyToolbox:arrayProduct:nlhs","Two outputs required.");
    }
    /* Make sure the first input argument is scalar. */
     if( !mxIsDouble(prhs[0]) || 
         mxIsComplex(prhs[0])) {
        mexErrMsgIdAndTxt("MyToolbox:arrayProduct:notDouble","First input scalar must be type double.");
    }
    
    /* Make sure the second input argument is type double. */
    if( !mxIsDouble(prhs[1]) || 
         mxIsComplex(prhs[1])) {
        mexErrMsgIdAndTxt("MyToolbox:arrayProduct:notDouble","Second input matrix must be type double.");
    }
    
    /* Check that number of rows in second input argument is 1. */
    if(mxGetM(prhs[1])!=1) {
        mexErrMsgIdAndTxt("MyToolbox:arrayProduct:notRowVector","Second input matrix must be a row vector.");
    }
    
     /* Make sure the third input argument is type double. */
    if( !mxIsDouble(prhs[2]) || 
         mxIsComplex(prhs[2])) {
        mexErrMsgIdAndTxt("MyToolbox:arrayProduct:notDouble","Third input matrix must be type double.");
    }
    
    /* Check that number of rows in third input argument is 1. */
    if(mxGetM(prhs[2])!=1) {
        mexErrMsgIdAndTxt("MyToolbox:arrayProduct:notRowVector","Third input matrix must be a row vector.");
    }

	
	
    /* Get the data of the first input.  */
    t = mxGetScalar(prhs[0]);

    /* Create a pointer to the real data in the second input matrix.  */
    x = mxGetPr(prhs[1]);

    /* Create a pointer to the real data in the third input matrix.  */
    x1 = mxGetPr(prhs[2]);
    
    /* Create the firts output matrix. */
    plhs[0] = mxCreateDoubleMatrix(1, DIMUC, mxREAL);

    /* Get a pointer to the real data in the first output matrix. */
    u = mxGetPr(plhs[0]);

	/* Create the second output matrix. */
	plhs[1] = mxCreateDoubleMatrix(1, DIMUC, mxREAL );

	/* Get a pointer to the real data in the second output matrix. */
	err = mxGetPr(plhs[1]);		


	/* Initialization */
	if(initialized == 0) {

		int nsim;	/* nsim is a global var */ 
		nsim  = (int)((tsim -tsim0) / ht);
	
		/* Global Variables */
		dimeq = dv * DIMUC;
		onemhdir = 1 - hdir / ht;
		hdirbht = hdir / ht; 
		onemzetahdir = 1 - zeta * hdir; 
		 
		/* Global Arrays */
		tau   = defvector(dv+1); 
		xtau  = defmatrix(dv+1, (int)DIMX);
		xtau1 = defmatrix(dv+1, (int)DIMX);
		ltau  = defmatrix(dv+1, (int)DIMX);
		utau  = defmatrix(dv, (int)DIMUC);
		utau1 = defmatrix(dv, (int)DIMUC); 
		hutau = defmatrix(dv, (int)DIMUC);
		hutau1= defmatrix(dv, (int)DIMUC);
		hutau2= defmatrix(dv, (int)DIMUC);
		x1s   = defvector(DIMX);
		bvec  = defvector(dimeq);
		duvec = defvector(dimeq);
		dutmp = defvector(dimeq);
		errvec= defvector(kmax+1);
		
		mmov(1, DIMX, x1, x0);		/* Initial value setting */
		init(x,u);
		printf("initialization \r\n");
	    mexAtExit(cLeanup);
	    initialized++;
    }
	
    /* Compute new input.
	 * Inputs t: previous time, x: previous state, x1: current state
	 * Output u: new input */
    unew(t,x,x1,u);

	/* Error norm */
	for( i = 0; i < DIMUC; i++ ){
		for( hu2 = 0, j=0; j < dv; j++ ){
			hu2 += hutau[j][i] * hutau[j][i];
		}
		err[i] = hu2;	
	}
}
